@@ -75,6 +75,50 @@ gconf_editor_window_close_window (gpointer callback_data, guint action, GtkWidge
 	gtk_object_destroy (GTK_OBJECT (window));
 }
 
+void
+gconf_editor_window_go_to (GConfEditorWindow *window,
+			   const char        *location)
+{
+	GtkTreePath *path, *child_path;
+	gint depth;
+	gint i;
+
+	g_return_if_fail (location != NULL);
+
+	child_path = gconf_tree_model_get_tree_path_from_gconf_path (
+				GCONF_TREE_MODEL (window->tree_model), location);
+	if (!child_path)
+		return;
+
+	path = gtk_tree_model_sort_convert_child_path_to_path (
+				GTK_TREE_MODEL_SORT (window->sorted_tree_model),
+				child_path);
+
+	gtk_tree_path_free (child_path);
+	
+	/* kind of hackish, but it works! */
+	depth = gtk_tree_path_get_depth (path);
+	for (i = 0; i < depth; i++) {
+		gint j;
+		GtkTreePath *cpath = gtk_tree_path_copy (path);
+
+		for (j = 0; j < (depth - i); j++)
+			gtk_tree_path_up (cpath);
+
+		gtk_tree_view_expand_row (GTK_TREE_VIEW (window->tree_view), cpath, FALSE);
+		gtk_tree_path_free (cpath);
+	}
+
+	gtk_tree_view_expand_row (GTK_TREE_VIEW (window->tree_view), path, FALSE);
+
+	gtk_tree_selection_select_path (
+			gtk_tree_view_get_selection (GTK_TREE_VIEW (window->tree_view)),
+			path);
+	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (window->tree_view),
+				      path, NULL, TRUE, 0.5, 0.5);
+	gtk_tree_path_free (path);
+}
+
 static void
 gconf_editor_window_copy_key_name (gpointer callback_data, guint action, GtkWidget *widget)
 {
@@ -360,14 +404,18 @@ gconf_editor_edit_key_response (GtkDialog *editor,
 		GConfValue *value;
 		GError     *error = NULL;
 		const char *path;
+		GConfClient *client;
 
 		value = gconf_key_editor_get_value (GCONF_KEY_EDITOR (editor));
 
 		path = gconf_key_editor_get_key_name (GCONF_KEY_EDITOR (editor));
 		g_assert (gconf_valid_key (path, NULL));
 
-		gconf_client_set (gconf_client_get_default (),
-				  path, value, &error);
+		client = gconf_client_get_default ();
+		/* if not writable we weren't allowed to change things anyway */
+		if (gconf_client_key_is_writable (client, path, &error))
+			gconf_client_set (gconf_client_get_default (),
+					  path, value, &error);
 
 		gconf_value_free (value);
 
@@ -375,7 +423,6 @@ gconf_editor_edit_key_response (GtkDialog *editor,
 			gconf_editor_window_popup_error_dialog (GTK_WINDOW (editor),
 								_("Could not change key value. Error message:\n%s"),
 								error);
-			g_error_free (error);
 			return;
 		}
 	}
@@ -405,7 +452,7 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 						 GTK_MESSAGE_INFO,
 						 GTK_BUTTONS_OK,
 						 _("Currently pairs and schemas can't "
-						   "be edited. This will be change in a later "
+						   "be edited. This will be changed in a later "
 						   "version."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
@@ -420,6 +467,10 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 	gconf_value_free (value);
 
 	gconf_key_editor_set_key_name (GCONF_KEY_EDITOR (editor), path);
+
+	gconf_key_editor_set_writable (GCONF_KEY_EDITOR (editor),
+				       gconf_client_key_is_writable (gconf_client_get_default (), path, NULL));
+
 	g_free (path);
 
 	g_signal_connect (editor, "response",
@@ -437,11 +488,96 @@ gconf_editor_window_list_view_row_activated (GtkTreeView       *tree_view,
 	gconf_editor_popup_window_edit_key (window, 0, NULL);
 }
 
+static void
+gconf_editor_popup_window_set_as_default (gpointer callback_data, guint action, GtkWidget *widget)
+{
+  GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (callback_data);
+  GtkWindow *window = GTK_WINDOW (callback_data);
+  static GConfEngine *defaults_engine = NULL;
+  static GConfClient *defaults_client = NULL;
+
+  GError *error = NULL;
+  GtkTreeIter iter;
+  GConfSchema *schema;
+  GConfValue *value;
+  GConfEntry *entry;
+  char *path = NULL;
+  const char *schema_path = NULL;
+  
+  if (defaults_engine == NULL) {
+    defaults_engine = gconf_engine_get_for_address (GCONF_DEFAULTS_SOURCE, &error);
+    if (error) {
+      gconf_editor_window_popup_error_dialog (window, _("Cannot create GConf engine. Error was:\n%s"), error);
+      return;
+    }
+  }
+  if (defaults_client == NULL) {
+    defaults_client = gconf_client_get_for_engine (defaults_engine);
+    if (defaults_client == NULL) {
+      /* TODO: error dialog */
+      g_print ("Could not create GConf client\n");
+      return;
+    }
+  }
+	
+  gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (gconfwindow->list_view)),
+                                   NULL, &iter);
+  gtk_tree_model_get (gconfwindow->sorted_list_model, &iter,
+                      GCONF_LIST_MODEL_KEY_PATH_COLUMN, &path,
+                      GCONF_LIST_MODEL_VALUE_COLUMN, &value,
+                      -1);
+
+  entry = gconf_client_get_entry (defaults_client, path, NULL, TRUE, &error);
+  if (error) {
+    gconf_editor_window_popup_error_dialog (window, _("Could not get schema location. Error was:\n%s"), error);
+    return;
+  }
+  g_assert (entry != NULL);
+
+  schema_path = gconf_entry_get_schema_name (entry);
+  if (schema_path == NULL) {
+    /* This key does not have an existing schema, so lets make one up. */
+    schema_path = g_strdup_printf ("/schemas%s", path);
+  }
+  schema = gconf_client_get_schema (defaults_client, schema_path, &error);
+  if (error) {
+    gconf_editor_window_popup_error_dialog (window, _("Could not get schema. Error was:\n%s"), error);
+    return;
+  }
+  if (schema == NULL) {
+    /* If we didn't get a schema it doesn't exist yet, so create one now. */
+    schema = gconf_schema_new ();
+  }
+
+  gconf_schema_set_default_value (schema, value);
+
+  if (!gconf_client_set_schema (defaults_client, schema_path, schema, &error)) {
+    gconf_editor_window_popup_error_dialog (window, _("Could not set schema. Error was:\n%s"), error);
+    return;
+  }
+
+  /* Docs say not to mix client and engine calls, but Havoc let me */
+  if (!gconf_engine_associate_schema (defaults_engine, path, schema_path, &error)) {
+    gconf_editor_window_popup_error_dialog (window, _("Could not associate schema. Error was:\n%s"), error);
+    return;
+  }
+
+  gconf_client_suggest_sync (defaults_client, &error);
+  if (error) {
+    gconf_editor_window_popup_error_dialog (window, _("Could not sync value. Error was:\n%s"), error);
+    return;
+  }
+  /* TODO: free the schema? */
+}
+
+
 static GtkItemFactoryEntry popup_menu_items[] = {
 	{ N_("/_New key..."), NULL, gconf_editor_popup_window_new_key,   0, "<StockItem>", GTK_STOCK_NEW },
 	{ N_("/Edit key..."), NULL, gconf_editor_popup_window_edit_key,  1, "<Item>", NULL }, 
 	{ N_("/sep1"),        NULL, 0,                                   0, "<Separator>" },
 	{ N_("/_Unset key"),  NULL, gconf_editor_popup_window_unset_key, 2, "<StockItem>", GTK_STOCK_DELETE },
+	{ N_("/sep2"),        NULL, 0,                                   0, "<Separator>" },
+	{ N_("/Set as _default"),  NULL, gconf_editor_popup_window_set_as_default, 3, "<Item>", NULL },
 };
 
 const char *image_popup_items_paths [] = {
@@ -537,6 +673,29 @@ gconf_editor_gconf_value_changed (GConfCellRenderer *cell, const gchar *path_str
 	g_free (key);
 }
 
+static gboolean
+gconf_editor_check_writable (GConfCellRenderer *cell, const gchar *path_str, GConfEditorWindow *window)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	gchar *key;
+	gboolean ret;
+	
+	path = gtk_tree_path_new_from_string (path_str);
+
+	gtk_tree_model_get_iter (window->sorted_list_model, &iter, path);
+
+	gtk_tree_model_get (window->sorted_list_model, &iter,
+			    GCONF_LIST_MODEL_KEY_PATH_COLUMN, &key,
+			    -1);
+
+	ret = gconf_client_key_is_writable (gconf_client_get_default (), key, NULL);
+
+	g_free (key);
+
+	return ret;
+}
+
 static void
 gconf_editor_window_list_view_popup_menu (GtkWidget *widget, GConfEditorWindow *window)
 {
@@ -605,6 +764,7 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 {
 	GtkTreeIter iter;
 	GConfSchema *schema;
+	GConfClient *client;
 	char *path;
 	
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
@@ -612,6 +772,7 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 		gtk_label_set_text (GTK_LABEL (window->owner_label), _("(None)"));
 		gtk_label_set_text (GTK_LABEL (window->short_desc_label), _("(None)"));
 		gtk_text_buffer_set_text (window->long_desc_buffer, _("(None)"), -1);
+		gtk_widget_hide (window->non_writable_label);
 		
 		return;
 	}
@@ -622,7 +783,14 @@ gconf_editor_window_update_list_selection (GtkTreeSelection *selection, GConfEdi
 	
 	gtk_label_set_text (GTK_LABEL (window->key_name_label), path);
 
-	schema = gconf_client_get_schema_for_key (gconf_client_get_default (), path);
+	client = gconf_client_get_default ();
+
+	if (gconf_client_key_is_writable (client, path, NULL))
+		gtk_widget_hide (window->non_writable_label);
+	else
+		gtk_widget_show (window->non_writable_label);
+
+	schema = gconf_client_get_schema_for_key (client, path);
 
 	if (schema != NULL && gconf_schema_get_long_desc (schema) != NULL) {
 		char *long_desc;
@@ -775,6 +943,13 @@ gconf_editor_window_setup_ui_prefs_handler (GConfEditorWindow *window)
 	g_object_unref (client);
 }
 
+static void
+gconf_editor_window_style_set (GtkWidget *widget, GtkStyle *prev_style, GtkWidget *text_view)
+{
+	gtk_widget_modify_base (text_view, GTK_STATE_NORMAL,
+				&GTK_WIDGET (widget)->style->bg[GTK_STATE_NORMAL]);
+}
+
 static void
 gconf_editor_window_finalize (GObject *object)
 {
@@ -817,7 +992,7 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	GtkAccelGroup *accel_group;      
 	GtkItemFactory *item_factory;
 	GtkTreePath *path;
-	GtkWidget *details_frame, *table, *label, *text_view;
+	GtkWidget *details_frame, *table, *label, *text_view, *image;
 	
 	/* Create popup menu */
 	window->popup_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", NULL);
@@ -852,7 +1027,10 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gtk_box_pack_start (GTK_BOX (vbox), gtk_item_factory_get_widget (item_factory, "<main>"), FALSE, FALSE, 0);
 
 	/* Hook up bookmarks */
-	gconf_bookmarks_hook_up_menu (window, gtk_item_factory_get_widget (item_factory, "/Bookmarks"));
+	gconf_bookmarks_hook_up_menu (window,
+				      gtk_item_factory_get_widget (item_factory, "/Bookmarks"),
+				      gtk_item_factory_get_widget (item_factory, "/Bookmarks/Add bookmark"),
+				      gtk_item_factory_get_widget (item_factory, "/Bookmarks/Edit bookmarks..."));
 
 	/* Create content area */
 	hpaned = gtk_hpaned_new ();
@@ -910,6 +1088,9 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	g_object_unref (G_OBJECT (window->list_model));
 	g_object_unref (G_OBJECT (window->sorted_list_model));
 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (window->sorted_list_model),
+					      GCONF_LIST_MODEL_KEY_NAME_COLUMN, GTK_SORT_ASCENDING);
+
 	g_signal_connect (window->list_view, "popup_menu",
 			  G_CALLBACK (gconf_editor_window_list_view_popup_menu), window);
 	g_signal_connect (window->list_view, "row_activated",
@@ -940,6 +1121,8 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	cell = gconf_cell_renderer_new ();
 	g_signal_connect (cell, "changed",
 			  G_CALLBACK (gconf_editor_gconf_value_changed), window);
+	g_signal_connect (cell, "check_writable",
+			  G_CALLBACK (gconf_editor_check_writable), window);
 	
 	window->value_column = column = gtk_tree_view_column_new_with_attributes (_("Value"),
 										  cell,
@@ -969,63 +1152,79 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	/* Create details area */
 	details_frame = gtk_frame_new (_("Key Documentation"));
 	gtk_paned_pack2 (GTK_PANED (vpaned), details_frame, FALSE, FALSE);
-
-	table = gtk_table_new (2, 4, FALSE);
+	
+	table = gtk_table_new (2, 5, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-
+	
 	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
 	gtk_container_add (GTK_CONTAINER (details_frame), table);
+
+
+	window->non_writable_label = gtk_hbox_new (FALSE, 6);
+	gtk_table_attach (GTK_TABLE (table), window->non_writable_label,
+			  0, 2, 0, 1,
+			  GTK_FILL, 0, 0, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (window->non_writable_label), image, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("This key is not writable"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (window->non_writable_label), label, FALSE, FALSE, 0);
+	
 	
 	label = gtk_label_new (_("Key Name:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-
+	
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 0, 1,
+			  0, 1, 1, 2,
 			  GTK_FILL, 0, 0, 0);
 	window->key_name_label = gtk_label_new (_("(None)"));
 	gtk_label_set_selectable (GTK_LABEL (window->key_name_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (window->key_name_label), 0.0, 0.5);	
 	gtk_table_attach (GTK_TABLE (table), window->key_name_label,
-			  1, 2, 0, 1,
+			  1, 2, 1, 2,
 			  GTK_FILL, 0, 0, 0);
-
+	
 	label = gtk_label_new (_("Key Owner:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);	
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 1, 2,
+			  0, 1, 2, 3,
 			  GTK_FILL, 0, 0, 0);
 	window->owner_label= gtk_label_new (_("(None)"));
 	gtk_label_set_selectable (GTK_LABEL (window->owner_label), TRUE);	
 	gtk_misc_set_alignment (GTK_MISC (window->owner_label), 0.0, 0.5);		
 	gtk_table_attach (GTK_TABLE (table), window->owner_label,
-			  1, 2, 1, 2,
+			  1, 2, 2, 3,
 			  GTK_FILL, 0, 0, 0);
 
 	label = gtk_label_new (_("Short Description:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);	
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 2, 3,
+			  0, 1, 3, 4,
 			  GTK_FILL, 0, 0, 0);
 	window->short_desc_label= gtk_label_new (_("(None)"));
 	gtk_label_set_line_wrap (GTK_LABEL (window->short_desc_label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (window->short_desc_label), TRUE);	
 	gtk_misc_set_alignment (GTK_MISC (window->short_desc_label), 0.0, 0.5);		
 	gtk_table_attach (GTK_TABLE (table), window->short_desc_label,
-			  1, 2, 2, 3,
+			  1, 2, 3, 4,
 			  GTK_FILL, 0, 0, 0);
 
 	label = gtk_label_new (_("Long Description:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.0);	
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 3, 4,
+			  0, 1, 4, 5,
 			  GTK_FILL, GTK_FILL, 0, 0);
 
 	window->long_desc_buffer = gtk_text_buffer_new (NULL);
 	gtk_text_buffer_set_text (window->long_desc_buffer, _("(None)"), -1);	
 	text_view = gtk_text_view_new_with_buffer (window->long_desc_buffer);
-	gtk_widget_modify_base (text_view, GTK_STATE_NORMAL,
-				&GTK_WIDGET (window)->style->bg[GTK_STATE_NORMAL]);
+	g_signal_connect (window, "style_set",
+			  G_CALLBACK (gconf_editor_window_style_set),
+			  text_view);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), FALSE);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -1035,11 +1234,12 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
 	
 	gtk_table_attach (GTK_TABLE (table), scrolled_window, 
-			  1, 2, 3, 4,
+			  1, 2, 4, 5,
 			  GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
 
 	gtk_widget_show_all (vbox);
-
+	gtk_widget_hide (window->non_writable_label);
+	
 	/* Set window icon */
 	icon = gdk_pixbuf_new_from_file (IMAGEDIR"/gconf-editor.png", NULL);
 	if (icon != NULL) {
