@@ -397,8 +397,8 @@ gconf_key_editor_class_init (GConfKeyEditorClass *klass)
 static void
 gconf_key_editor_init (GConfKeyEditor *editor)
 {
-	GtkWidget *hbox, *frame;
-	GtkWidget *label;
+	GtkWidget *hbox, *frame, *framebox;
+	GtkWidget *label, *image;
 	GtkWidget *value_box;
 	GtkWidget *button_box, *button;
 	GtkSizeGroup *size_group;
@@ -451,13 +451,28 @@ gconf_key_editor_init (GConfKeyEditor *editor)
 	gtk_widget_show_all (hbox);
 	gtk_frame_set_label_widget (GTK_FRAME (frame), hbox);
 
+	framebox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (framebox);
+	gtk_container_add (GTK_CONTAINER (frame), framebox);
+
+	editor->non_writable_label = gtk_hbox_new (FALSE, 3);
+	gtk_box_pack_start (GTK_BOX (framebox), editor->non_writable_label, FALSE, FALSE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), image, FALSE, FALSE, 3);
+
+	label = gtk_label_new (_("This key is not writable"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), label, FALSE, FALSE, 3);
+
 	editor->notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (editor->notebook),
 				    FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (editor->notebook),
 				      FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (editor->notebook), 3);
-	gtk_container_add (GTK_CONTAINER (frame), editor->notebook);
+	gtk_box_pack_start (GTK_BOX (framebox), editor->notebook, FALSE, FALSE, 0);
 
 	editor->active_type = EDIT_INTEGER;
 
@@ -793,6 +808,17 @@ gconf_key_editor_set_key_name (GConfKeyEditor *editor, const char *path)
 	gtk_entry_set_text (GTK_ENTRY (editor->name_entry), path);
 }
 
+void
+gconf_key_editor_set_writable (GConfKeyEditor *editor, gboolean writable)
+{
+	if (writable)
+		gtk_widget_hide (editor->non_writable_label);
+	else
+		gtk_widget_show (editor->non_writable_label);
+
+	gtk_widget_set_sensitive (editor->notebook, writable);
+}
+
 G_CONST_RETURN char *
 gconf_key_editor_get_key_name (GConfKeyEditor *editor)
 {
